# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.http import HttpResponse, JsonResponse
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.decorators import login_required
from django.contrib.auth import get_user_model

from .api import imgur_uploader
from .settings import MARTOR_MARKDOWNIFY_FUNCTION
from .utils import LazyEncoder

User = get_user_model()


def markdownfy_view(request):
    if request.method == "POST":
        content = request.POST.get("content", "")
        markdownify = import_string(MARTOR_MARKDOWNIFY_FUNCTION)
        return HttpResponse(markdownify(content))
    return HttpResponse(_("Invalid request!"))


@login_required
def markdown_imgur_uploader(request):
    """
    Makdown image upload for uploading to imgur.com
    and represent as json to markdown editor.
    """

    def is_ajax(request):
        return request.META.get("HTTP_X_REQUESTED_WITH") == "XMLHttpRequest"

    if request.method == "POST" and is_ajax(request):
        if "markdown-image-upload" in request.FILES:
            image = request.FILES["markdown-image-upload"]
            response_data = imgur_uploader(image=image)
            return HttpResponse(response_data, content_type="application/json")
        return HttpResponse(_("Invalid request!"))
    return HttpResponse(_("Invalid request!"))


@login_required
def markdown_search_user(request):
    """
    Json usernames of the users registered & actived.

    url(method=get):
        /martor/search-user/?username={username}

    Response:
        error:
            - `status` is status code (204)
            - `error` is error message.
        success:
            - `status` is status code (204)
            - `data` is list dict of usernames.
                { 'status': 200,
                  'data': [
                    {'usernane': 'john'},
                    {'usernane': 'albert'}]
                }
    """
    response_data = {}
    username = request.GET.get("username")

    if username is not None and username != "" and " " not in username:

        queries = {"%s__icontains" % User.USERNAME_FIELD: username}
        users = User.objects.filter(**queries).filter(is_active=True)
        if users.exists():
            usernames = list(users.values_list("username", flat=True))
            response_data.update({"status": 200, "data": usernames})
            return JsonResponse(response_data)

        error_message = _(
            "No users registered as `%(username)s` " "or user is unactived."
        )
        error_message = error_message % {"username": username}
        response_data.update({"status": 204, "error": error_message})
    else:
        error_message = _("Validation Failed for field `username`")
        response_data.update({"status": 204, "error": error_message})

    return JsonResponse(response_data, encoder=LazyEncoder)



import django

from .views import TestFormView


if django.VERSION >= (2, 0):
    from django.urls import path, include

    urlpatterns = [
        path("test-form-view/", TestFormView.as_view()),
        path("martor/", include("martor.urls")),
    ]
else:
    from django.conf.urls import url, include

    urlpatterns = [
        url(r"^test-form-view/$", TestFormView.as_view()),
        url(r"^martor/", include("martor.urls")),
    ]



import markdown
from django.contrib.auth import get_user_model
from ..settings import MARTOR_ENABLE_CONFIGS, MARTOR_MARKDOWN_BASE_MENTION_URL

"""
>>> import markdown
>>> md = markdown.Markdown(extensions=['martor.utils.extensions.mention'])
>>> md.convert('@[summonagus]')
'<p><a class="direct-mention-link"
href="https://example.com/profile/summonagus/">summonagus</a></p>'
>>>
>>> md.convert('hello @[summonagus], i mentioned you!')
'<p>hello <a class="direct-mention-link"
href="https://example.com/profile/summonagus/">summonagus</a>,
i mentioned you!</p>'
>>>
"""

MENTION_RE = r"(?<!\!)\@\[([^\]]+)\]"


class MentionPattern(markdown.inlinepatterns.Pattern):
    def handleMatch(self, m):
        username = self.unescape(m.group(2))
        users = get_user_model().objects.filter(
            username=username, is_active=True
        )  # noqa: E501

        """Makesure `username` is registered and actived."""
        if MARTOR_ENABLE_CONFIGS["mention"] == "true":
            if users.exists():
                url = "{0}{1}/".format(
                    MARTOR_MARKDOWN_BASE_MENTION_URL, username
                )  # noqa: E501
                el = markdown.util.etree.Element("a")
                el.set("href", url)
                el.set("class", "direct-mention-link")
                el.text = markdown.util.AtomicString("@" + username)
                return el


class MentionExtension(markdown.Extension):
    def extendMarkdown(self, md, md_globals):
        """Setup `mention_link` with MentionPattern"""
        md.inlinePatterns["mention_link"] = MentionPattern(MENTION_RE, md)


def makeExtension(*args, **kwargs):
    return MentionExtension(*args, **kwargs)


if __name__ == "__main__":
    import doctest

    doctest.testmod()



"""
Source: https://github.com/r0wb0t/markdown-urlize

A more liberal autolinker
Inspired by Django's urlize function.

Positive examples:

>>> import markdown
>>> md = markdown.Markdown(extensions=['urlize'])

>>> md.convert('http://example.com/')
u'<p><a href="http://example.com/">http://example.com/</a></p>'

>>> md.convert('go to http://example.com')
u'<p>go to <a href="http://example.com">http://example.com</a></p>'

>>> md.convert('example.com')
u'<p><a href="http://example.com">example.com</a></p>'

>>> md.convert('example.net')
u'<p><a href="http://example.net">example.net</a></p>'

>>> md.convert('www.example.us')
u'<p><a href="http://www.example.us">www.example.us</a></p>'

>>> md.convert('(www.example.us/path/?name=val)')
u'<p>(<a href="http://www.example.us/path/?name=val">www.example.us/path/?name=val</a>)</p>'

>>> md.convert('go to <http://example.com> now!')
u'<p>go to <a href="http://example.com">http://example.com</a> now!</p>'

Negative examples:

>>> md.convert('del.icio.us')
u'<p>del.icio.us</p>'

"""

import markdown

# Global Vars
URLIZE_RE = "(%s)" % "|".join(
    [
        r"<(?:f|ht)tps?://[^>]*>",
        r"\b(?:f|ht)tps?://[^)<>\s]+[^.,)<>\s]",
        r"\bwww\.[^)<>\s]+[^.,)<>\s]",
        r"[^(<\s]+\.(?:com|net|org)\b",
    ]
)


class UrlizePattern(markdown.inlinepatterns.Pattern):
    """Return a link Element given an autolink (`http://example/com`)."""

    def handleMatch(self, m):
        url = m.group(2)

        if url.startswith("<"):
            url = url[1:-1]

        text = url

        if not url.split("://")[0] in ("http", "https", "ftp"):
            if "@" in url and "/" not in url:
                url = "mailto:" + url
            else:
                url = "http://" + url

        el = markdown.util.etree.Element("a")
        el.set("href", url)
        el.text = markdown.util.AtomicString(text)
        return el


class UrlizeExtension(markdown.Extension):
    """Urlize Extension for Python-Markdown."""

    def extendMarkdown(self, md, md_globals):
        """Replace autolink with UrlizePattern"""
        md.inlinePatterns["autolink"] = UrlizePattern(URLIZE_RE, md)


def makeExtension(*args, **kwargs):
    return UrlizeExtension(*args, **kwargs)


if __name__ == "__main__":
    import doctest

    doctest.testmod()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings

# Choices are: "semantic", "bootstrap"
MARTOR_THEME = getattr(settings, "MARTOR_THEME", "bootstrap")

# Global martor settings
# Input: string boolean, `true/false`
MARTOR_ENABLE_CONFIGS = getattr(
    settings,
    "MARTOR_ENABLE_CONFIGS",
    {
        "emoji": "true",  # enable/disable emoji icons.
        "imgur": "true",  # enable/disable imgur/custom uploader.
        "mention": "false",  # enable/disable mention
        "jquery": "true",  # include/revoke jquery (require for admin django)
        "living": "false",  # enable/disable live updates in preview
        "spellcheck": "false",  # enable/disable spellcheck in form textareas
        "hljs": "true",  # enable/disable hljs highlighting in preview
    },
)

# To show the toolbar buttons
MARTOR_TOOLBAR_BUTTONS = getattr(
    settings,
    "MARTOR_TOOLBAR_BUTTONS",
    [
        "bold",
        "italic",
        "horizontal",
        "heading",
        "pre-code",
        "blockquote",
        "unordered-list",
        "ordered-list",
        "link",
        "image-link",
        "image-upload",
        "emoji",
        "direct-mention",
        "toggle-maximize",
        "help",
    ],
)

# To setup the martor editor with title label or not (default is False)
MARTOR_ENABLE_LABEL = getattr(settings, "MARTOR_ENABLE_LABEL", False)

# Imgur API Keys
MARTOR_IMGUR_CLIENT_ID = getattr(settings, "MARTOR_IMGUR_CLIENT_ID", "")
MARTOR_IMGUR_API_KEY = getattr(settings, "MARTOR_IMGUR_API_KEY", "")

# Markdownify
MARTOR_MARKDOWNIFY_FUNCTION = getattr(
    settings, "MARTOR_MARKDOWNIFY_FUNCTION", "martor.utils.markdownify"
)
MARTOR_MARKDOWNIFY_URL = getattr(
    settings, "MARTOR_MARKDOWNIFY_URL", "/martor/markdownify/"
)

# Markdown extensions
MARTOR_MARKDOWN_EXTENSIONS = getattr(
    settings,
    "MARTOR_MARKDOWN_EXTENSIONS",
    [
        "markdown.extensions.extra",
        "markdown.extensions.nl2br",
        "markdown.extensions.smarty",
        "markdown.extensions.fenced_code",
        # Custom markdown extensions.
        "martor.extensions.urlize",
        "martor.extensions.del_ins",  # ~~strikethrough~~ and ++underscores++
        "martor.extensions.mention",  # to parse markdown mention
        "martor.extensions.emoji",  # to parse markdown emoji
        "martor.extensions.mdx_video",  # to parse embed/iframe video
        "martor.extensions.escape_html",  # to handle the XSS vulnerabilities
    ],
)

# Markdown Extensions Configs
MARTOR_MARKDOWN_EXTENSION_CONFIGS = getattr(
    settings, "MARTOR_MARKDOWN_EXTENSION_CONFIGS", {}
)

# Markdown urls
MARTOR_UPLOAD_URL = getattr(
    settings, "MARTOR_UPLOAD_URL", "/martor/uploader/"  # for imgur
)
MARTOR_SEARCH_USERS_URL = getattr(
    settings, "MARTOR_SEARCH_USERS_URL", "/martor/search-user/"  # for mention
)

# Markdown Extensions
MARTOR_MARKDOWN_BASE_EMOJI_URL = getattr(
    settings,
    "MARTOR_MARKDOWN_BASE_EMOJI_URL",
    "https://github.githubassets.com/images/icons/emoji/",
)

MARTOR_MARKDOWN_BASE_MENTION_URL = getattr(
    settings,
    "MARTOR_MARKDOWN_BASE_MENTION_URL",
    "https://python.web.id/author/",
)

# If you need to use your own themed "bootstrap" or "semantic ui" dependency
# replace the values with the file in your static files dir
MARTOR_ALTERNATIVE_JS_FILE_THEME = getattr(
    settings, "MARTOR_ALTERNATIVE_JS_FILE_THEME", None
)
MARTOR_ALTERNATIVE_CSS_FILE_THEME = getattr(
    settings, "MARTOR_ALTERNATIVE_CSS_FILE_THEME", None
)
MARTOR_ALTERNATIVE_JQUERY_JS_FILE = getattr(
    settings, "MARTOR_ALTERNATIVE_JQUERY_JS_FILE", None
)

# URL schemes that are allowed within links
ALLOWED_URL_SCHEMES = getattr(
    settings,
    "ALLOWED_URL_SCHEMES",
    [
        "file",
        "ftp",
        "ftps",
        "http",
        "https",
        "irc",
        "mailto",
        "sftp",
        "ssh",
        "tel",
        "telnet",
        "tftp",
        "vnc",
        "xmpp",
    ],
)

# https://gist.github.com/mrmrs/7650266
ALLOWED_HTML_TAGS = getattr(
    settings,
    "ALLOWED_HTML_TAGS",
    [
        "a",
        "abbr",
        "b",
        "blockquote",
        "br",
        "cite",
        "code",
        "command",
        "dd",
        "del",
        "dl",
        "dt",
        "em",
        "fieldset",
        "h1",
        "h2",
        "h3",
        "h4",
        "h5",
        "h6",
        "hr",
        "i",
        "iframe",
        "img",
        "input",
        "ins",
        "kbd",
        "label",
        "legend",
        "li",
        "ol",
        "optgroup",
        "option",
        "p",
        "pre",
        "small",
        "span",
        "strong",
        "sub",
        "sup",
        "table",
        "tbody",
        "td",
        "tfoot",
        "th",
        "thead",
        "tr",
        "u",
        "ul",
    ],
)

# https://github.com/decal/werdlists/blob/master/html-words/html-attributes-list.txt
ALLOWED_HTML_ATTRIBUTES = getattr(
    settings,
    "ALLOWED_HTML_ATTRIBUTES",
    [
        "alt",
        "class",
        "color",
        "colspan",
        # "data",
        "datetime",
        "height",
        "href",
        "id",
        "name",
        "reversed",
        "rowspan",
        "scope",
        "src",
        "style",
        "title",
        "type",
        "width",
    ],
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms
from django.template.loader import get_template
from django.contrib.admin import widgets

from .settings import (
    MARTOR_THEME,
    MARTOR_ENABLE_CONFIGS,
    MARTOR_UPLOAD_URL,
    MARTOR_MARKDOWNIFY_URL,
    MARTOR_SEARCH_USERS_URL,
    MARTOR_MARKDOWN_BASE_EMOJI_URL,
    MARTOR_TOOLBAR_BUTTONS,
    MARTOR_ALTERNATIVE_JS_FILE_THEME,
    MARTOR_ALTERNATIVE_CSS_FILE_THEME,
    MARTOR_ALTERNATIVE_JQUERY_JS_FILE,
)


def get_theme():
    """function to get the selected theme"""
    supported_themes = ["bootstrap", "semantic"]
    if MARTOR_THEME in supported_themes:
        return MARTOR_THEME
    return "bootstrap"


class MartorWidget(forms.Textarea):
    def render(self, name, value, attrs=None, renderer=None, **kwargs):
        # Make the settings the default attributes to pass
        attributes_to_pass = {
            "data-enable-configs": MARTOR_ENABLE_CONFIGS,
            "data-upload-url": MARTOR_UPLOAD_URL,
            "data-markdownfy-url": MARTOR_MARKDOWNIFY_URL,
            "data-search-users-url": MARTOR_SEARCH_USERS_URL,
            "data-base-emoji-url": MARTOR_MARKDOWN_BASE_EMOJI_URL,
        }

        # Make sure that the martor value is in the class attr passed in
        if "class" in attrs:
            attrs["class"] += " martor"
        else:
            attrs["class"] = "martor"

        # Update and overwrite with the attributes passed in
        attributes_to_pass.update(attrs)

        # Update and overwrite with any attributes that are on the widget
        # itself. This is also the only way we can push something in without
        # being part of the render chain.
        attributes_to_pass.update(self.attrs)

        template = get_template("martor/%s/editor.html" % get_theme())
        emoji_enabled = MARTOR_ENABLE_CONFIGS.get("emoji") == "true"
        mentions_enabled = MARTOR_ENABLE_CONFIGS.get("mention") == "true"

        widget = super().render(name, value, attributes_to_pass)

        return template.render(
            {
                "martor": widget,
                "field_name": name,
                "emoji_enabled": emoji_enabled,
                "mentions_enabled": mentions_enabled,
                "toolbar_buttons": MARTOR_TOOLBAR_BUTTONS,
            }
        )

    class Media:
        selected_theme = get_theme()
        css = {
            "all": (
                "plugins/css/ace.min.css",
                "plugins/css/resizable.min.css",
                "martor/css/martor.%s.min.css" % selected_theme,
                "martor/css/martor-admin.min.css",
            )
        }
        js = (
            "plugins/js/ace.js",
            "plugins/js/mode-markdown.js",
            "plugins/js/ext-language_tools.js",
            "plugins/js/theme-github.js",
            "plugins/js/highlight.min.js",
            "plugins/js/resizable.min.js",
            "plugins/js/emojis.min.js",
            "martor/js/martor.%s.min.js" % selected_theme,
        )

        # Adding the following scripts to the end
        # of the tuple in case it affects behaviour.
        # spellcheck configuration
        if MARTOR_ENABLE_CONFIGS.get("spellcheck") == "true":
            js = ("plugins/js/typo.js", "plugins/js/spellcheck.js").__add__(js)

        # support alternative vendor theme file like: bootstrap, semantic)
        # 1. vendor css theme
        if MARTOR_ALTERNATIVE_CSS_FILE_THEME:
            css_theme = MARTOR_ALTERNATIVE_CSS_FILE_THEME
            css["all"] = (css_theme,).__add__(css.get("all"))
        else:
            css_theme = "plugins/css/%s.min.css" % selected_theme
            css["all"] = (css_theme,).__add__(css.get("all"))

        # 2. vendor js theme
        if MARTOR_ALTERNATIVE_JS_FILE_THEME:
            js_theme = MARTOR_ALTERNATIVE_JS_FILE_THEME
            js = (MARTOR_ALTERNATIVE_JS_FILE_THEME,).__add__(js)
        else:
            js_theme = "plugins/js/%s.min.js" % selected_theme
            js = (js_theme,).__add__(js)

        # 3. vendor jQUery
        if MARTOR_ALTERNATIVE_JQUERY_JS_FILE:
            js = (MARTOR_ALTERNATIVE_JQUERY_JS_FILE,).__add__(js)
        elif MARTOR_ENABLE_CONFIGS.get("jquery") == "true":
            js = ("plugins/js/jquery.min.js",).__add__(js)


class AdminMartorWidget(MartorWidget, widgets.AdminTextareaWidget):
    pass



import markdown

from ..settings import MARTOR_MARKDOWN_BASE_EMOJI_URL as EMOJI_URL

"""
>>> import markdown
>>> md = markdown.Markdown(extensions=['martor.utils.extensions.emoji'])
>>> md.convert(':smile:')
'<p><img class="marked-emoji" src="http://example.com/emoji/smile.png" /></p>'
>>>
"""

EMOJI_RE = r"(:[+\-\w]+:)"
EMOJIS = [
    ":yellow_heart:",
    ":blue_heart:",
    ":purple_heart:",
    ":heart:",
    ":green_heart:",
    ":broken_heart:",
    ":heartbeat:",
    ":heartpulse:",
    ":two_hearts:",
    ":revolving_hearts:",
    ":cupid:",
    ":sparkling_heart:",
    ":sparkles:",
    ":star:",
    ":star2:",
    ":dizzy:",
    ":boom:",
    ":collision:",
    ":anger:",
    ":exclamation:",
    ":question:",
    ":grey_exclamation:",
    ":grey_question:",
    ":zzz:",
    ":dash:",
    ":sweat_drops:",
    ":notes:",
    ":musical_note:",
    ":fire:",
    ":thumbsup:",
    ":-1:",
    ":thumbsdown:",
    ":ok_hand:",
    ":punch:",
    ":facepunch:",
    ":fist:",
    ":v:",
    ":wave:",
    ":hand:",
    ":raised_hand:",
    ":open_hands:",
    ":point_up:",
    ":point_down:",
    ":point_left:",
    ":point_right:",
    ":raised_hands:",
    ":pray:",
    ":point_up_2:",
    ":clap:",
    ":muscle:",
    ":nail_care:",
    ":sunny:",
    ":umbrella:",
    ":cloud:",
    ":snowflake:",
    ":zap:",
    ":cyclone:",
    ":foggy:",
    ":ocean:",
    ":bouquet:",
    ":cherry_blossom:",
    ":tulip:",
    ":four_leaf_clover:",
    ":rose:",
    ":sunflower:",
    ":hibiscus:",
    ":maple_leaf:",
    ":leaves:",
    ":fallen_leaf:",
    ":herb:",
    ":mushroom:",
    ":cactus:",
    ":palm_tree:",
    ":evergreen_tree:",
    ":deciduous_tree:",
    ":chestnut:",
    ":seedling:",
    ":blossom:",
    ":ear_of_rice:",
    ":shell:",
    ":globe_with_meridians:",
    ":new_moon:",
    ":waxing_crescent_moon:",
    ":first_quarter_moon:",
    ":waxing_gibbous_moon:",
    ":full_moon:",
    ":waning_gibbous_moon:",
    ":last_quarter_moon:",
    ":waning_crescent_moon:",
    ":last_quarter_moon_with_face:",
    ":first_quarter_moon_with_face:",
    ":crescent_moon:",
    ":earth_africa:",
    ":earth_americas:",
    ":earth_asia:",
    ":volcano:",
    ":milky_way:",
    ":partly_sunny:",
    ":bamboo:",
    ":gift_heart:",
    ":school_satchel:",
    ":mortar_board:",
    ":flags:",
    ":fireworks:",
    ":sparkler:",
    ":wind_chime:",
    ":rice_scene:",
    ":gift:",
    ":bell:",
    ":no_bell:",
    ":tanabata_tree:",
    ":tada:",
    ":confetti_ball:",
    ":balloon:",
    ":crystal_ball:",
    ":cd:",
    ":dvd:",
    ":floppy_disk:",
    ":camera:",
    ":video_camera:",
    ":movie_camera:",
    ":computer:",
    ":tv:",
    ":iphone:",
    ":phone:",
    ":telephone:",
    ":telephone_receiver:",
    ":pager:",
    ":fax:",
    ":minidisc:",
    ":vhs:",
    ":sound:",
    ":speaker:",
    ":mute:",
    ":loudspeaker:",
    ":mega:",
    ":hourglass:",
    ":hourglass_flowing_sand:",
    ":alarm_clock:",
    ":watch:",
    ":radio:",
    ":satellite:",
    ":loop:",
    ":mag:",
    ":mag_right:",
    ":unlock:",
    ":lock:",
    ":lock_with_ink_pen:",
    ":closed_lock_with_key:",
    ":key:",
    ":bulb:",
    ":flashlight:",
    ":high_brightness:",
    ":low_brightness:",
    ":electric_plug:",
    ":battery:",
    ":calling:",
    ":email:",
    ":mailbox:",
    ":postbox:",
    ":bath:",
    ":bathtub:",
    ":shower:",
    ":toilet:",
    ":wrench:",
    ":nut_and_bolt:",
    ":hammer:",
    ":seat:",
    ":moneybag:",
    ":yen:",
    ":dollar:",
    ":pound:",
    ":euro:",
    ":credit_card:",
    ":e-mail:",
    ":money_with_wings:",
    ":inbox_tray:",
    ":outbox_tray:",
    ":envelope:",
    ":incoming_envelope:",
    ":postal_horn:",
    ":mailbox_closed:",
    ":mailbox_with_mail:",
    ":mailbox_with_no_mail:",
    ":package:",
    ":door:",
    ":smoking:",
    ":bomb:",
    ":gun:",
    ":hocho:",
    ":pill:",
    ":syringe:",
    ":page_facing_up:",
    ":page_with_curl:",
    ":bookmark_tabs:",
    ":bar_chart:",
    ":chart_with_upwards_trend:",
    ":chart_with_downwards_trend:",
    ":scroll:",
    ":clipboard:",
    ":calendar:",
    ":date:",
    ":card_index:",
    ":file_folder:",
    ":open_file_folder:",
    ":scissors:",
    ":pushpin:",
    ":paperclip:",
    ":black_nib:",
    ":pencil2:",
    ":straight_ruler:",
    ":triangular_ruler:",
    ":closed_book:",
    ":green_book:",
    ":blue_book:",
    ":orange_book:",
    ":notebook:",
    ":notebook_with_decorative_cover:",
    ":ledger:",
    ":books:",
    ":bookmark:",
    ":name_badge:",
    ":microscope:",
    ":telescope:",
    ":newspaper:",
    ":football:",
    ":basketball:",
    ":soccer:",
    ":baseball:",
    ":tennis:",
    ":8ball:",
    ":rugby_football:",
    ":bowling:",
    ":golf:",
    ":gem:",
    ":ring:",
    ":trophy:",
    ":musical_score:",
    ":musical_keyboard:",
    ":violin:",
    ":video_game:",
    ":flower_playing_cards:",
    ":game_die:",
    ":dart:",
    ":clapper:",
    ":memo:",
    ":pencil:",
    ":book:",
    ":art:",
    ":microphone:",
    ":headphones:",
    ":trumpet:",
    ":saxophone:",
    ":guitar:",
    ":shoe:",
    ":sandal:",
    ":high_heel:",
    ":lipstick:",
    ":boot:",
    ":shirt:",
    ":tshirt:",
    ":necktie:",
    ":womans_clothes:",
    ":dress:",
    ":running_shirt_with_sash:",
    ":jeans:",
    ":kimono:",
    ":ribbon:",
    ":tophat:",
    ":crown:",
    ":womans_hat:",
    ":mans_shoe:",
    ":briefcase:",
    ":closed_umbrella:",
    ":handbag:",
    ":pouch:",
    ":purse:",
    ":eyeglasses:",
    ":fishing_pole_and_fish:",
    ":coffee:",
    ":tea:",
    ":sake:",
    ":baby_bottle:",
    ":beer:",
    ":beers:",
    ":cocktail:",
    ":tropical_drink:",
    ":wine_glass:",
    ":fork_and_knife:",
    ":pizza:",
    ":hamburger:",
    ":fries:",
    ":poultry_leg:",
    ":meat_on_bone:",
    ":spaghetti:",
    ":curry:",
    ":fried_shrimp:",
    ":bento:",
    ":sushi:",
    ":fish_cake:",
    ":rice_ball:",
    ":rice_cracker:",
    ":rice:",
    ":ramen:",
    ":stew:",
    ":oden:",
    ":dango:",
    ":egg:",
    ":bread:",
    ":doughnut:",
    ":custard:",
    ":icecream:",
    ":ice_cream:",
    ":shaved_ice:",
    ":birthday:",
    ":cake:",
    ":cookie:",
    ":chocolate_bar:",
    ":candy:",
    ":lollipop:",
    ":honey_pot:",
    ":apple:",
    ":green_apple:",
    ":tangerine:",
    ":lemon:",
    ":cherries:",
    ":grapes:",
    ":watermelon:",
    ":strawberry:",
    ":peach:",
    ":melon:",
    ":banana:",
    ":pear:",
    ":pineapple:",
    ":sweet_potato:",
    ":eggplant:",
    ":tomato:",
    ":corn:",
    ":house:",
    ":house_with_garden:",
    ":school:",
    ":office:",
    ":post_office:",
    ":hospital:",
    ":bank:",
    ":convenience_store:",
    ":love_hotel:",
    ":hotel:",
    ":wedding:",
    ":church:",
    ":department_store:",
    ":european_post_office:",
    ":city_sunrise:",
    ":city_sunset:",
    ":japanese_castle:",
    ":european_castle:",
    ":tent:",
    ":factory:",
    ":tokyo_tower:",
    ":japan:",
    ":mount_fuji:",
    ":sunrise_over_mountains:",
    ":sunrise:",
    ":stars:",
    ":statue_of_liberty:",
    ":bridge_at_night:",
    ":carousel_horse:",
    ":rainbow:",
    ":ferris_wheel:",
    ":fountain:",
    ":roller_coaster:",
    ":ship:",
    ":speedboat:",
    ":boat:",
    ":sailboat:",
    ":rowboat:",
    ":anchor:",
    ":rocket:",
    ":airplane:",
    ":helicopter:",
    ":steam_locomotive:",
    ":tram:",
    ":mountain_railway:",
    ":bike:",
    ":aerial_tramway:",
    ":suspension_railway:",
    ":mountain_cableway:",
    ":tractor:",
    ":blue_car:",
    ":oncoming_automobile:",
    ":car:",
    ":red_car:",
    ":taxi:",
    ":oncoming_taxi:",
    ":articulated_lorry:",
    ":bus:",
    ":oncoming_bus:",
    ":rotating_light:",
    ":police_car:",
    ":oncoming_police_car:",
    ":fire_engine:",
    ":ambulance:",
    ":minibus:",
    ":truck:",
    ":train:",
    ":station:",
    ":train2:",
    ":bullettrain_front:",
    ":bullettrain_side:",
    ":light_rail:",
    ":monorail:",
    ":railway_car:",
    ":trolleybus:",
    ":ticket:",
    ":fuelpump:",
    ":vertical_traffic_light:",
    ":traffic_light:",
    ":warning:",
    ":construction:",
    ":beginner:",
    ":atm:",
    ":slot_machine:",
    ":busstop:",
    ":barber:",
    ":hotsprings:",
    ":checkered_flag:",
    ":crossed_flags:",
    ":izakaya_lantern:",
    ":moyai:",
    ":circus_tent:",
    ":performing_arts:",
    ":round_pushpin:",
    ":triangular_flag_on_post:",
    ":jp:",
    ":kr:",
    ":cn:",
    ":us:",
    ":fr:",
    ":es:",
    ":it:",
    ":ru:",
    ":gb:",
    ":uk:",
    ":de:",
    ":one:",
    ":two:",
    ":three:",
    ":four:",
    ":five:",
    ":six:",
    ":seven:",
    ":eight:",
    ":nine:",
    ":keycap_ten:",
    ":1234:",
    ":zero:",
    ":hash:",
    ":symbols:",
    ":arrow_backward:",
    ":arrow_down:",
    ":arrow_forward:",
    ":arrow_left:",
    ":capital_abcd:",
    ":abcd:",
    ":abc:",
    ":arrow_lower_left:",
    ":arrow_lower_right:",
    ":arrow_right:",
    ":arrow_up:",
    ":arrow_upper_left:",
    ":arrow_upper_right:",
    ":arrow_double_down:",
    ":arrow_double_up:",
    ":arrow_down_small:",
    ":arrow_heading_down:",
    ":arrow_heading_up:",
    ":leftwards_arrow_with_hook:",
    ":arrow_right_hook:",
    ":left_right_arrow:",
    ":arrow_up_down:",
    ":arrow_up_small:",
    ":arrows_clockwise:",
    ":arrows_counterclockwise:",
    ":rewind:",
    ":fast_forward:",
    ":information_source:",
    ":ok:",
    ":twisted_rightwards_arrows:",
    ":repeat:",
    ":repeat_one:",
    ":new:",
    ":top:",
    ":up:",
    ":cool:",
    ":free:",
    ":ng:",
    ":cinema:",
    ":koko:",
    ":signal_strength:",
    ":u5272:",
    ":u5408:",
    ":u55b6:",
    ":u6307:",
    ":u6708:",
    ":u6709:",
    ":u6e80:",
    ":u7121:",
    ":u7533:",
    ":u7a7a:",
    ":u7981:",
    ":sa:",
    ":restroom:",
    ":mens:",
    ":womens:",
    ":baby_symbol:",
    ":no_smoking:",
    ":parking:",
    ":wheelchair:",
    ":metro:",
    ":baggage_claim:",
    ":accept:",
    ":wc:",
    ":potable_water:",
    ":put_litter_in_its_place:",
    ":secret:",
    ":congratulations:",
    ":m:",
    ":passport_control:",
    ":left_luggage:",
    ":customs:",
    ":ideograph_advantage:",
    ":cl:",
    ":sos:",
    ":id:",
    ":no_entry_sign:",
    ":underage:",
    ":no_mobile_phones:",
    ":do_not_litter:",
    ":non-potable_water:",
    ":no_bicycles:",
    ":no_pedestrians:",
    ":children_crossing:",
    ":no_entry:",
    ":eight_spoked_asterisk:",
    ":sparkle:",
    ":eight_pointed_black_star:",
    ":heart_decoration:",
    ":vs:",
    ":vibration_mode:",
    ":mobile_phone_off:",
    ":chart:",
    ":currency_exchange:",
    ":aries:",
    ":taurus:",
    ":gemini:",
    ":cancer:",
    ":leo:",
    ":virgo:",
    ":libra:",
    ":scorpius:",
    ":sagittarius:",
    ":capricorn:",
    ":aquarius:",
    ":pisces:",
    ":ophiuchus:",
    ":six_pointed_star:",
    ":negative_squared_cross_mark:",
    ":a:",
    ":b:",
    ":ab:",
    ":o2:",
    ":diamond_shape_with_a_dot_inside:",
    ":recycle:",
    ":end:",
    ":back:",
    ":on:",
    ":soon:",
    ":clock1:",
    ":clock130:",
    ":clock10:",
    ":clock1030:",
    ":clock11:",
    ":clock1130:",
    ":clock12:",
    ":clock1230:",
    ":clock2:",
    ":clock230:",
    ":clock3:",
    ":clock330:",
    ":clock4:",
    ":clock430:",
    ":clock5:",
    ":clock530:",
    ":clock6:",
    ":clock630:",
    ":clock7:",
    ":clock730:",
    ":clock8:",
    ":clock830:",
    ":clock9:",
    ":clock930:",
    ":heavy_dollar_sign:",
    ":copyright:",
    ":registered:",
    ":tm:",
    ":x:",
    ":heavy_exclamation_mark:",
    ":bangbang:",
    ":interrobang:",
    ":o:",
    ":heavy_multiplication_x:",
    ":heavy_plus_sign:",
    ":heavy_minus_sign:",
    ":heavy_division_sign:",
    ":white_flower:",
    ":100:",
    ":heavy_check_mark:",
    ":ballot_box_with_check:",
    ":radio_button:",
    ":link:",
    ":curly_loop:",
    ":wavy_dash:",
    ":part_alternation_mark:",
    ":trident:",
    ":black_small_square:",
    ":white_small_square:",
    ":black_medium_small_square:",
    ":white_medium_small_square:",
    ":black_medium_square:",
    ":white_medium_square:",
    ":white_large_square:",
    ":white_check_mark:",
    ":black_square_button:",
    ":white_square_button:",
    ":black_circle:",
    ":white_circle:",
    ":red_circle:",
    ":large_blue_circle:",
    ":large_blue_diamond:",
    ":large_orange_diamond:",
    ":small_blue_diamond:",
    ":small_orange_diamond:",
    ":small_red_triangle:",
    ":small_red_triangle_down:",
]


class EmojiPattern(markdown.inlinepatterns.Pattern):
    def handleMatch(self, m):
        emoji = self.unescape(m.group(2))
        if emoji not in EMOJIS:
            return emoji

        url = "{0}{1}.png".format(EMOJI_URL, emoji.replace(":", ""))
        el = markdown.util.etree.Element("img")
        el.set("src", url)
        el.set("class", "marked-emoji")
        el.text = markdown.util.AtomicString(emoji)
        return el


class EmojiExtension(markdown.Extension):
    def extendMarkdown(self, md, md_globals):
        """Setup `emoji_img` with EmojiPattern"""
        md.inlinePatterns["emoji_img"] = EmojiPattern(EMOJI_RE, md)


def makeExtension(*args, **kwargs):
    return EmojiExtension(*args, **kwargs)


if __name__ == "__main__":
    import doctest

    doctest.testmod()



from django.test import TestCase, override_settings
from django.contrib.auth.models import User
from martor.utils import markdownify


class SimpleTest(TestCase):
    def setUp(self):
        self.user_password = "TestEgg@1234"
        self.user = User.objects.create_user(
            username="user1",
            email="user1@mail.com",
            password=self.user_password,
        )
        self.client.login(
            username=self.user.username,
            password=self.user_password,
        )

    def test_form(self):
        response = self.client.get("/test-form-view/")
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed("test_form_view.html")
        self.assertContains(response, "main-martor-description")
        self.assertContains(response, "main-martor-wiki")

    @override_settings(
        MARTOR_ENABLE_CONFIGS={
            "emoji": "true",  # enable/disable emoji icons.
            "imgur": "true",  # enable/disable imgur/custom uploader.
            "mention": "true",  # enable/disable mention
            "jquery": "true",  # include/revoke jquery (require for admin django) # noqa: E501
            "living": "false",  # enable/disable live updates in preview
            "spellcheck": "false",  # enable/disable spellcheck in form textareas # noqa: E501
            "hljs": "true",  # enable/disable hljs highlighting in preview
        }
    )
    def test_markdownify(self):
        # Heading
        response = self.client.post(
            "/martor/markdownify/", {"content": "# Hello world!"}
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.content.decode("utf-8"), "<h1>Hello world!</h1>"
        )  # noqa: E501

        # Link
        response = self.client.post(
            "/martor/markdownify/",
            {"content": "[python](https://python.web.id)"},
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.content.decode("utf-8"),
            '<p><a href="https://python.web.id">python</a></p>',
        )  # noqa: E501

        # Image
        response = self.client.post(
            "/martor/markdownify/",
            {"content": "![image](https://imgur.com/test.png)"},
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.content.decode("utf-8"),
            '<p><img alt="image" src="https://imgur.com/test.png"></p>',
        )  # noqa: E501

        # # Mention
        # response = self.client.post(
        #     '/martor/markdownify/',
        #     {'content': f'@[{self.user.username}]'}
        # )
        # self.assertEqual(response.status_code, 200)
        # self.assertEqual(
        #     response.content.decode('utf-8'),
        #     '...fixme'
        # )

    def test_markdownify_xss_handled(self):
        xss_payload_1 = "[aaaa](javascript:alert(1))"
        response_1 = markdownify(xss_payload_1)
        self.assertEqual(response_1, '<p><a href=":">aaaa</a></p>')

        xss_payload_2 = '![" onerror=alert(1) ](x)'
        response_2 = markdownify(xss_payload_2)
        self.assertEqual(
            response_2, '<p><img alt="&quot; onerror=alert(1) " src="x"></p>'
        )

        xss_payload_3 = '[xss](" onmouseover=alert(document.domain) l)'
        response_3 = markdownify(xss_payload_3)
        self.assertEqual(
            response_3,
            '<p><a href="&quot; onmouseover=alert(document.domain)">xss</a>)</p>',  # noqa: E501
        )
